@@ -29,7 +29,7 @@
 #
 #########################################################################
 #
-# $Id: install.pl,v 1.223 2005/03/12 17:13:44 mbr Exp $
+# $Id: install.pl,v 1.233 2005/10/26 02:21:12 mbr Exp $
 #
 
 use Cwd;
@@ -86,30 +86,37 @@ my %required_perl_modules = (
     'Unix::Syslog' => {
         'version' => '0.100',
         'force-lib-install' => 0,
+        'mod-dir' => 'Unix-Syslog'
     },
     'Bit::Vector' => {
         'version' => '6.3',
         'force-lib-install' => 0,
+        'mod-dir' => 'Bit-Vector'
     },
     'Date::Calc', => {
         'version' => '5.3',
         'force-lib-install' => 0,
+        'mod-dir' => 'Date-Calc'
     },
     'Net::IPv4Addr' => {
         'version' => '0.10',
         'force-lib-install' => 0,
+        'mod-dir' => 'Net-IPv4Addr'
     },
     'IPTables::Parse' => {
         'version' => '0.2',
         'force-lib-install' => 1,
+        'mod-dir' => 'IPTables-Parse'
     },
     'IPTables::ChainMgr' => {
-        'version' => '0.1',
+        'version' => '0.2',
         'force-lib-install' => 1,
+        'mod-dir' => 'IPTables-ChainMgr'
     },
     'Psad' => {
         'version' => '1.4.1',
         'force-lib-install' => 1,
+        'mod-dir' => 'Psad'
     }
 );
 
@@ -153,13 +160,17 @@ unless ($found) {
 my $noarchive   = 0;
 my $uninstall   = 0;
 my $help        = 0;
+my $skip_syslog_test = 0;
+my $cmdline_force_install = 0;
 my $syslog_conf = '';
 
 &usage(1) unless (GetOptions(
-    'no-preserve'   => \$noarchive,   ### Don't preserve existing configs.
-    'syslog-conf=s' => \$syslog_conf, ### specify path to syslog config file.
-    'uninstall'     => \$uninstall,   ### Uninstall psad.
-    'help'          => \$help         ### Display help.
+    'force-mod-install' => \$cmdline_force_install,  ### force install of all modules
+    'no-preserve'       => \$noarchive,   ### Don't preserve existing configs.
+    'syslog-conf=s'     => \$syslog_conf, ### specify path to syslog config file.
+    'no-syslog-test'    => \$skip_syslog_test,
+    'uninstall'         => \$uninstall,   ### Uninstall psad.
+    'help'              => \$help         ### Display help.
 ));
 &usage(0) if $help;
 
@@ -564,42 +575,44 @@ sub install() {
     &put_string('SYSLOG_DAEMON', $syslog_str,
         "${PSAD_CONFDIR}/psad.conf");
 
-    my $restarted_syslog = 0;
-    if ($syslog_str eq 'syslogd') {
-        if (-e $syslog_conf) {
-            &append_fifo_syslog($syslog_conf);
-            if (((system "$Cmds{'killall'} -HUP syslogd 2> /dev/null")>>8) == 0) {
-                &logr("[+] HUP signal sent to syslogd.\n");
+    if ($syslog_str ne 'ulogd') {
+        my $restarted_syslog = 0;
+        if ($syslog_str eq 'syslogd') {
+            if (-e $syslog_conf) {
+                &append_fifo_syslog($syslog_conf);
+                if (((system "$Cmds{'killall'} -HUP syslogd 2> /dev/null")>>8) == 0) {
+                    &logr("[+] HUP signal sent to syslogd.\n");
+                    $restarted_syslog = 1;
+                }
+            }
+        } elsif ($syslog_str eq 'syslog-ng') {
+            if (-e $syslog_conf) {
+                &append_fifo_syslog_ng($syslog_conf);
+                if (((system "$Cmds{'killall'} -HUP syslog-ng 2> /dev/null")>>8) == 0) {
+                    &logr("[+] HUP signal sent to syslog-ng.\n");
+                    $restarted_syslog = 1;
+                }
+            }
+        } elsif ($syslog_str eq 'metalog') {
+            if (-e $syslog_conf) {
+                &config_metalog($syslog_conf);
+                &logr("[-] Metalog support is shaky in psad.  " .
+                    "Use at your own risk.\n");
+                ### don't send warning about not restarting metalog daemon
                 $restarted_syslog = 1;
             }
         }
-    } elsif ($syslog_str eq 'syslog-ng') {
-        if (-e $syslog_conf) {
-            &append_fifo_syslog_ng($syslog_conf);
-            if (((system "$Cmds{'killall'} -HUP syslog-ng 2> /dev/null")>>8) == 0) {
-                &logr("[+] HUP signal sent to syslog-ng.\n");
-                $restarted_syslog = 1;
-            }
-        }
-    } elsif ($syslog_str eq 'metalog') {
-        if (-e $syslog_conf) {
-            &config_metalog($syslog_conf);
-            &logr("[-] Metalog support is shaky in psad.  " .
-                "Use at your own risk.\n");
-            ### don't send warning about not restarting metalog daemon
-            $restarted_syslog = 1;
+
+        unless ($restarted_syslog) {
+            &logr("[-] Could not restart any syslog daemons.\n");
         }
     }
 
-    unless ($restarted_syslog) {
-        &logr("[-] Could not restart any syslog daemons.\n");
-    }
-
-    if (-x $Cmds{'iptables'}) {
+    if (-x $Cmds{'iptables'} and not $skip_syslog_test) {
         &logr("[+] Found iptables. Testing syslog configuration:\n");
         ### make sure we actually see packets being logged by
         ### the firewall.
-        &test_syslog_config($syslog_str);
+        &test_syslog_config($syslog_str) if $syslog_str ne 'ulogd';
     }
 
     ### see if there are any "_CHANGEME_" strings left and give the
@@ -862,19 +875,27 @@ sub install_perl_module() {
         unless defined $required_perl_modules{$mod_name}{'version'};
     die '[*] Missing force-lib-install key in required_perl_modules hash.'
         unless defined $required_perl_modules{$mod_name}{'force-lib-install'};
+    die '[*] Missing mod-dir key in required_perl_modules hash.'
+        unless defined $required_perl_modules{$mod_name}{'mod-dir'};
 
     my $version = $required_perl_modules{$mod_name}{'version'};
 
     my $install_module = 0;
 
-    if ($required_perl_modules{$mod_name}{'force-lib-install'}) {
+    if ($required_perl_modules{$mod_name}{'force-lib-install'}
+            or $cmdline_force_install) {
         ### install regardless of whether the module may already be
         ### installed
         $install_module = 1;
-    } elsif (not has_perl_module($mod_name)) {
-        ### install the module in the /usr/lib/psad directory because
-        ### it is not already installed.
-        $install_module = 1;
+    } else {
+        if (has_perl_module($mod_name)) {
+            print "[+] Module $mod_name is already installed in the ",
+                "system perl tree, skipping.\n";
+        } else {
+            ### install the module in the /usr/lib/fwknop directory because
+            ### it is not already installed.
+            $install_module = 1;
+        }
     }
 
     if ($install_module) {
@@ -882,9 +903,9 @@ sub install_perl_module() {
             &logr("[+] Creating $LIBDIR\n");
             mkdir $LIBDIR, 0755 or die "[*] Could not mkdir $LIBDIR: $!";
         }
-        &logr("[+] Installing the $mod_name $version perl module\n");
-        my $mod_dir = $mod_name;
-        $mod_dir =~ s/::/-/g;
+        &logr("[+] Installing the $mod_name $version perl " .
+            "module in $LIBDIR/\n");
+        my $mod_dir = $required_perl_modules{$mod_name}{'mod-dir'};
         chdir $mod_dir or die "[*] Could not chdir to ",
             "$mod_dir: $!";
         unless (-e 'Makefile.PL') {
@@ -1326,7 +1347,6 @@ sub test_syslog_config() {
 
     ### remove any "test_DROP" lines from fwdata file and ipt_prefix_ctr
     ### before seeing if new ones can be written
-    &scrub_fwdata();
     &scrub_prefix_ctr();
 
     my $start_kmsgsd = 1;
@@ -1358,6 +1378,10 @@ sub test_syslog_config() {
     open FWDATA, "${PSAD_DIR}/fwdata" or
         die "[*] Could not open ${PSAD_DIR}/fwdata: $!";
 
+    ### sleep to give kmsgsd a chance to pick up the packet
+    ### log message from syslog
+    seek FWDATA,0,2;  ### seek to the end of the file
+
     ### try to connect to $test_port to generate an iptables
     ### drop message.  Note that since nothing is listening on
     ### the port we will immediately receive a tcp reset.
@@ -1368,11 +1392,10 @@ sub test_syslog_config() {
         'Timeout'  => 1
     );
     undef $sock if defined $sock;
-
-    ### sleep to give kmsgsd a chance to pick up the packet
-    ### log message from syslog
     sleep 2;
+
     my $found = 0;
+
     my @pkts = <FWDATA>;
     close FWDATA;
     for my $pkt (@pkts) {
@@ -1383,7 +1406,9 @@ sub test_syslog_config() {
     system "$Cmds{'iptables'} -D INPUT 1";
 
     ### remove the any new test_DROP lines we just created
-    &scrub_fwdata();
+    ### (this probably is not necessary because psad is not
+    ### running).
+    &scrub_prefix_ctr();
 
     if ($found) {
         &logr("[+] Successful $syslog_str reconfiguration.\n\n");
@@ -1405,23 +1430,6 @@ sub test_syslog_config() {
     return;
 }
 
-sub scrub_fwdata() {
-    if (-e "${PSAD_DIR}/fwdata") {
-        open SCRUB, "< ${PSAD_DIR}/fwdata" or
-            die "[*] Could not open ${PSAD_DIR}/fwdata: $!";
-        my @lines = <SCRUB>;
-        close SCRUB;
-
-        open SCRUB, "> ${PSAD_DIR}/fwdata" or
-            die "[*] Could not open ${PSAD_DIR}/fwdata: $!";
-        for my $line (@lines) {
-            print SCRUB $line unless $line =~ /test_DROP/;
-        }
-        close SCRUB;
-    }
-    return;
-}
-
 sub scrub_prefix_ctr() {
     if (-e "${PSAD_DIR}/ipt_prefix_ctr") {
         open SCRUB, "< ${PSAD_DIR}/ipt_prefix_ctr" or
@@ -1646,13 +1654,16 @@ sub query_email() {
 }
 
 sub query_syslog() {
-    &logr("\n[+] psad supports the syslogd, syslog-ng, and metalog logging " .
-        "daemons.\n\n");
+    &logr("\n[+] psad supports the syslogd, syslog-ng, ulogd, and\n" .
+        "    metalog logging daemons.  Which system logger is running?\n\n");
     my $ans = '';
-    while ($ans ne 'syslogd' and $ans ne 'syslog-ng' and $ans ne 'metalog') {
-        &logr("    Which system logger is running (syslogd, " .
-            "syslog-ng, metalog)?  ");
+    while ($ans ne 'syslogd' and $ans ne 'syslog-ng' and $ans ne 'ulogd'
+            and $ans ne 'metalog') {
+        &logr("    syslogd / syslog-ng / ulogd / metalog? [syslogd] ");
         $ans = <STDIN>;
+        if ($ans eq "\n") {  ### allow default to take over
+            $ans = 'syslogd';
+        }
         $ans =~ s/\s*//g;
 
         if ($ans eq 'syslogd') {
@@ -1665,7 +1676,7 @@ sub query_syslog() {
             ### allow command line --syslog-conf arg to take over
             $syslog_conf = '/etc/metalog/metalog.conf' unless $syslog_conf;
         }
-        if ($syslog_conf and not -e $syslog_conf) {
+        if ($ans ne 'ulogd' and $syslog_conf and not -e $syslog_conf) {
             die
 "[-] The config file $syslog_conf does not exist. Re-run install.pl\n",
 "    with the --syslog-conf argument to specify the path to the syslog\n",
@@ -1676,6 +1687,7 @@ sub query_syslog() {
         unless ($ans and
             ($ans eq 'syslogd'
             or $ans eq 'syslog-ng'
+            or $ans eq 'ulogd'
             or $ans eq 'metalog'));
     print "\n";
     return $ans;
@@ -1909,11 +1921,17 @@ sub usage() {
         my $exitcode = shift;
         print <<_HELP_;
 
-Usage: install.pl [-n] [-u] [-h]
+Usage: install.pl [-f] [-s <file> ] [-u] [--no-syslog-test]
+                  [--no-preserve] [-h]
 
-    -n  --no-preserve   - disable preservation of old configs.
-    -u  --uninstall     - uninstall psad.
-    -h  --help          - prints this help message.
+    -u  --uninstall     - Uninstall psad.
+    -f  --force-mod     - Force all perl modules to be installed
+                          even if some already exist in the system
+                          /usr/lib/perl5 tree.
+    -s  <file>          - Specify path to syslog.conf file.
+    --no-syslog-test    - Skip syslog reconfiguration test.
+    --no-preserve       - Disable preservation of old configs.
+    -h  --help          - Prints this help message.
 
 _HELP_
         exit $exitcode;
