@@ -29,7 +29,7 @@
 *     USA
 ******************************************************************************
 *
-*  $Id: kmsgsd.c,v 1.47 2005/03/08 05:50:37 mbr Exp $
+*  $Id: kmsgsd.c,v 1.50 2005/10/13 03:44:40 mbr Exp $
 */
 
 /* includes */
@@ -131,7 +131,7 @@ int main(int argc, char *argv[]) {
     /* open the psadfifo named pipe.  Note that we are opening the pipe
      * _without_ the O_NONBLOCK flag since we want the read on the file
      * descriptor to block until there is something new in the pipe.
-     * Also, not that we are opening with O_RDWR, since this seems to
+     * Also, note that we are opening with O_RDWR, since this seems to
      * fix the problem with kmsgsd not blocking on the read() if the
      * system logger dies (and hence closes its file descriptor for the
      * psadfifo). */
@@ -217,17 +217,18 @@ int main(int argc, char *argv[]) {
 #endif
             }
 #ifdef DEBUG
-        if (matched_ipt_log_msg) {
-            puts(buf);
-            fprintf(stderr, "[+] Line matched search strings.\n");
-            fwlinectr++;
-            if (fwlinectr % 50 == 0)
-                fprintf(stderr,
-                    "[+] Processed %d firewall lines.\n", fwlinectr);
-            matched_ipt_log_msg = 0;
-        } else {
-            puts(buf);
-            printf("[-] Line did not match search strings.\n");
+            if (matched_ipt_log_msg) {
+                puts(buf);
+                fprintf(stderr, "[+] Line matched search strings.\n");
+                fwlinectr++;
+                if (fwlinectr % 50 == 0)
+                    fprintf(stderr,
+                        "[+] Processed %d firewall lines.\n", fwlinectr);
+                matched_ipt_log_msg = 0;
+            } else {
+                puts(buf);
+                printf("[-] Line did not match search strings.\n");
+            }
 #endif
         }
     }
@@ -339,6 +340,14 @@ static void parse_fw_search_file(char *fw_search_file)
             }
         }
     }
+    if (! fw_search_all_flag && num_fw_search_strings == 0) {
+        /* there are no FW_MSG_SEARCH vars in fw_search.conf; default
+         * to "DROP".  Psad will generate a syslog warning.  */
+        fw_msg_search[num_fw_search_strings]
+            = (char *) malloc(strlen("DROP"));
+        strlcpy(fw_msg_search[0], "DROP", MAX_GEN_LEN);
+        num_fw_search_strings++;
+    }
     fclose(fw_search_ptr);
     return;
 }
